#!/usr/bin/env ruby
#
# $Id: Ci.rb,v 1.6 2003/01/19 12:22:15 Yuukis Exp $
#
# Set your configuration.

Root = "#{ENV['HOME']}/Ci/"
ScriptRoot = "#{ENV['HOME']}/projects/Ci/"
Gzip = '/usr/bin/gzip'

#
#

CiVersion = 'alpha.3'
CiCode = 'X-10'

require 'gtk'

Dir.mkdir(Root) unless File.exist?(Root)
BBSMenuURI = 'http://www.ff.iij4u.or.jp/~ch2/bbsmenu.html'
MonazillaUA = "Monazilla/1.00 (C.i./#{CiVersion})"
Zcat = "#{Gzip} -d"

load "#{ScriptRoot}util2ch.rb"
CiPixRoot = "#{ScriptRoot}pixmap/"

Color_DarkGreen = Gdk::Color.new(0,25700,0)
Color_DarkBlue = Gdk::Color.new(0,0,35723)
Color_Purple = Gdk::Color.new(32896,0,32896)
Color_Blue = Gdk::Color.new(0,0,65535)

RegexID = /^\d/
RegexFollowMany = /^(.*?)([>]+\s*\d+([\-*|,*]\d*)*)(.*)/e
RegexFollowOne = /^(.*?)([>]+\s*\d+)(.*)/e
RegexQuote = /^\s*[>]\s?.*/e
RegexUri = /^(.*?)(h?ttps?:[a-zA-Z0-9%&\?\/\;\:\@\&\=\+\$\,\-\_\.\!\~\*\'\\#]+)(.*)$/

class NodeData
	def initialize(uri)
		@uri = uri
	end

	def uri
		@uri
	end
end

class Status
	def initialize
		@status = false
	end

	def toggleStatus
		if @status == true
			@status = false
		else
			@status = true
		end
		return(@status)
	end
end

class CiPix
    def initialize(w)
		@images = {}
		pixes = Dir.glob("#{CiPixRoot}*.xpm")
		pixes.each {|pixmap|
			pix, mask = Gdk::Pixmap.create_from_xpm(w.window, nil, pixmap)
			pixmap.sub(/\/.*\/(.*)\.xpm/) { pixmap = $1 }
			@images[pixmap]= Gtk::Pixmap.new(pix, mask)
		}
	end

	def getPixmap(name)
		return(@images[name])
	end
end

class CiPost
	def initialize
		@post = PostMessage2ch.new
		@server = ''
		@board = ''
		@dat = ''
		postBox
	end

	def postBox
		@w = Gtk::Window.new
		@w.set_default_size(350, 300)
		@w.signal_connect('delete_event') { close }
		
		nameLabel = Gtk::Label.new('Name')
		mailLabel = Gtk::Label.new('Mailto')

		@nameEntry = Gtk::Entry.new
		@mailEntry = Gtk::Entry.new

		labelBox = Gtk::VBox.new
		labelBox.pack_start nameLabel
		labelBox.pack_start mailLabel

		entryBox = Gtk::VBox.new
		entryBox.pack_start @nameEntry
		entryBox.pack_start @mailEntry

		entriesBox = Gtk::HBox.new
		entriesBox.pack_start labelBox, false
		entriesBox.pack_start entryBox, true

		@text = Gtk::Text.new.set_editable(true)

		postButton = Gtk::Button.new('Post')
			postButton.signal_connect('clicked') {
				wrote, body = @post.postMessage(@server, @board, @dat,
					@nameEntry.get_text, @mailEntry.get_text,
					@text.get_chars(0, @text.get_length))
				close if wrote
				setStatusText(body)
				}
		cancelButton = Gtk::Button.new('Cancel')
			cancelButton.signal_connect('clicked') { close }
		buttonBox = Gtk::HBox.new
		buttonBox.pack_start postButton
		buttonBox.pack_start cancelButton

		@status = Gtk::Label.new('')

		postBox = Gtk::VBox.new
		postBox.pack_start entriesBox, false
		postBox.pack_start @text, true
		postBox.pack_start buttonBox, false
		postBox.pack_start @status, false

		@w.add postBox
	end

	def setStatusText(str)
		@status.set_text(str)
	end

	def setWindowTitle(title, server, board, dat)
		@w.set_title('Re: '+title)
		@server = server
		@board = board
		@dat = dat
	end

	def open
		@w.show_all
	end

	def close
		@nameEntry.delete_text(0, @nameEntry.get_text.length)
		@mailEntry.delete_text(0, @mailEntry.get_text.length)
		@text.delete_text(0, @text.get_length)
		@w.hide
	end
end

class Ci
	def initialize(widget)
		@top = widget
		@ci = Util2ch.new
		@postBox = CiPost.new
		@board = @ci.getProfile
		@bbsmenu = @ci.parseBBSMenu
		@backPageNum = 0  # textbacksubject / bookmark / finder Τɤ뤫
		@title = ''
		@dat = ''
		@updateFilter = Status.new
		@bookmarkFilter = Status.new
		@ci.getBookmark(@board)
	end

	def setTooltip(widget, text)
		tooltip = Gtk::Tooltips.new
		tooltip.set_tip(widget, text, nil)
	end

	def exitCi
		@ci.setProfile(@board)
		@ci.writeBookmark(@board)
	end

	def onFreeze
		cur = Gdk::Cursor.new(Gdk::Cursor::Constants::WATCH)
		@top.window.set_cursor(cur)
	end

	def offFreeze
		cur = Gdk::Cursor.new(Gdk::Cursor::Constants::LEFT_PTR)
		@top.window.set_cursor(cur)
	end

	def subjectScrolledWindowCreate
		sw = Gtk::ScrolledWindow.new
		sw.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS)
		sw.set_usize(200, 200)
		return(sw)
	end

	def subjectCListCreate
		subjects = Gtk::CList.new(['S', 'C', '', 'dat', 'F', 'C', 'subject'])
		subjects.set_column_justification(0, Gtk::JUSTIFY_RIGHT)
		subjects.set_column_justification(1, Gtk::JUSTIFY_RIGHT)
		subjects.set_column_justification(2, Gtk::JUSTIFY_RIGHT)
		subjects.set_column_justification(3, Gtk::JUSTIFY_RIGHT)
		subjects.set_column_justification(4, Gtk::JUSTIFY_RIGHT)
		subjects.set_column_justification(5, Gtk::JUSTIFY_RIGHT)
		subjects.set_column_justification(6, Gtk::JUSTIFY_LEFT)
		subjects.set_column_auto_resize(0, true)
		subjects.set_column_auto_resize(1, true)
		subjects.set_column_auto_resize(2, true)
		subjects.set_column_auto_resize(3, true)
		subjects.set_column_min_width(4, 30)
		subjects.set_column_auto_resize(4, true)
		subjects.set_column_min_width(5, 30)
		subjects.set_column_auto_resize(5, true)
		subjects.set_column_auto_resize(6, true)
		return(subjects)
	end

	def boardSelecter  # CTree褫CListCListΤۤå票⡣
		sw = Gtk::ScrolledWindow.new
			sw.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS)

		@boardTree = Gtk::CTree.new(["Board Name"], 0)
		@boardTree.set_line_style(Gtk::CTree::LINES_DOTTED)
			root = @boardTree.insert_node(nil, nil, ["The 2ch Boards"], 1,
				nil, nil, nil, nil, false, true)
			nodes = @bbsmenu
			nodes.each {|info|
				if info[1] == 'C-'
					@semiRoot = @boardTree.insert_node(root, nil, [info[0]], 1,
						nil, nil, nil, nil, false, false)
				else
					node = @boardTree.insert_node(@semiRoot, nil, [info[0]], 1,
						nil, nil, nil, nil, true, false)
					@boardTree.node_set_row_data(node, NodeData.new(info[1]))
				end
			}

			@boardTree.signal_connect('tree_select_row') {|t, node, c|
				if @boardTree.get_node_info(node)[6]
					@ci.writeBookmark(@board)
					@server, @board = @ci.parseUri(@boardTree.node_get_row_data(node).uri)
					subjectFrameUpdate
					@ci.getBookmark(@board)
				end
			}
		sw.add @boardTree
		return(sw)
	end

	def finder
		@findEntry = Gtk::Entry.new
		@findEntry.set_editable(true)
			@findEntry.signal_connect('activate') { subjectFrameUpdate(false, true) }
		findButton = Gtk::Button.new(' Find ')
			findButton.signal_connect('clicked') { subjectFrameUpdate(false, true) }
		closeButton = Gtk::Button.new(' Close ')
			closeButton.signal_connect('clicked') {
				@findEntry.delete_text(0, @findEntry.get_text.length)
				subjectFrameUpdate(false)
				@topNote.set_page(0)
				@mainNote.set_page(0)
			}

		box = Gtk::HBox.new
		box.pack_start @findEntry, true
		box.pack_start findButton, false
		box.pack_start closeButton, false
		return(box)
	end

	def topNote
		@topNote = Gtk::Notebook.new
			@topNote.set_show_tabs(false)
			@topNote.append_page(finder, Gtk::Label.new('Finder'))
		return(@topNote)
	end

	def topNoteSetPage(num)
		@topNote.set_page(num)
	end

	def mainNote
		@mainNote = Gtk::Notebook.new
			# @mainNote.set_scrollable(true)
			@mainNote.set_show_tabs(false)
			@mainNote.append_page(subjectFrame, Gtk::Label.new('subjectFrame'))
			#@mainNote.append_page(bookmarkFrame, Gtk::Label.new('bookmarkFrame'))
			#@mainNote.append_page(finderFrame, Gtk::Label.new('finderFrame'))
			@mainNote.append_page(textFrame, Gtk::Label.new('textFrame'))
		return(@mainNote)
	end

	def mainNoteSetPage(num)
		@mainNote.set_page(num)
	end

	def urlView
		@urlview = Gtk::Entry.new
		@urlview.set_editable(false)
		@urlview.set_text('Welcome to C.i. World !!')
		return(@urlview)
	end

	def subjectFrame
		sw = subjectScrolledWindowCreate
		@subjects = subjectCListCreate
			@subjects.signal_connect('select_row') {|w, r, c, e|
				onFreeze
				@backPageNum = 0
				textFrameThreadGet(@board, @subjects.get_text(r, 6), @subjects.get_text(r, 3))
				@mainNote.set_page(1)
				offFreeze
			}

			@subjects.signal_connect('click_column') {|w, c|
				case c
				when 0
					onFreeze
					@subjects.clear
					if @updateFilter.toggleStatus
						h = @ci.getSubjects(@board, false, false, true)
					else
						h = @ci.getSubjects(@board, false, false, false)
					end
					@subjects.freeze
					h.each {|th|
						@subjects.append([th[0], th[1], th[2], th[3], th[4], th[5], th[6]])
					}
					@subjects.thaw
					offFreeze
				end
			}

			h = @ci.getSubjects(@board)
			@subjects.freeze
			h.each {|th|
				@subjects.append([th[0], th[1], th[2], th[3], th[4], th[5], th[6]])
			}
			@subjects.thaw
		sw.add @subjects
		return(sw)
	end

	def subjectFrameUpdate(get = true, search = false, bookmark = false)
		onFreeze
		@subjects.clear
		@subjects.freeze
		if search
			a = @ci.getSubjects(@board, false, @findEntry.get_text)
		elsif bookmark
			a = @ci.getSubjects(@board, false, false, false, true)
		else
			a = @ci.getSubjects(@board, get)
		end
		a.each {|th|
			@subjects.append([th[0], th[1], th[2], th[3], th[4], th[5], th[6]])
		}
		@subjects.thaw
		offFreeze
	end

	def bookmarkControl(board, title, dat)
		if @ci.bookmark?(dat)
			@ci.deleteBookmark(dat)
			return(' add BookMark ')
		else
			@ci.addBookmark(dat)
			return(' delete BookMark ')
		end
		#onFreeze
		#bookmarkFrameUpdate
		#offFreeze
	end

	def textFrame
		textBox = Gtk::VBox.new
		buttons = Gtk::HBox.new
			@subjectLabel = Gtk::Label.new(' No Title ')
			blank = Gtk::Label.new
			backButton = Gtk::Button.new.add Pix.getPixmap('back')
			setTooltip(backButton, 'Back')
			backButton.signal_connect('clicked') {
				setThreadCached
				setVadjInfo(@board, @dat, @vadj.value)
				@mainNote.set_page(@backPageNum)
				@subjects.unselect_all  # bookmarkξunselectʤɤ褦
				#subjectFrameUpdate(false)
				@urlview.set_text("#{@board} Board at #{@ci.server(@board)}")
			}
			followupButton = Gtk::Button.new.add Pix.getPixmap('followup')
			setTooltip(followupButton, 'Follow Up')
			followupButton.signal_connect('clicked') { @postBox.open }
			@bookmarkButton = Gtk::Button.new(' BookMark ')
			@bookmarkButton.signal_connect('clicked') {
				@bookmarkButton.child.set_text(bookmarkControl(@board, @title, @dat))
			}
					
		buttons.pack_start @subjectLabel, false
		buttons.pack_start blank
		buttons.pack_start @bookmarkButton, false
		buttons.pack_start backButton, false
		buttons.pack_start followupButton, false

		text = Gtk::HBox.new
			@vadj = Gtk::Adjustment.new(0, 0, 0, 0, 0, 0)
			sb = Gtk::VScrollbar.new(@vadj)
			@texts = Gtk::Text.new(nil, @vadj)
			@texts.signal_connect('key_press_event'){|w, e|  # from goRua. Thanks.
				case e.keyval
				when Gdk::GDK_space
					tmp_vadj = @vadj.value + @vadj.page_size/1.1
					@vadj.set_value(tmp_vadj)
				when Gdk::GDK_BackSpace
					tmp_vadj = @vadj.value - @vadj.page_size/1.1
					@vadj.set_value(tmp_vadj)
				when Gdk::GDK_g
					@vadj.set_value(0)
					#@vadj.lower
				when Gdk::GDK_G
					@vadj.set_value(1000000)
					#@vadj.upper
				when Gdk::GDK_q
					backButton.clicked
				end
			}
		text.pack_start @texts
		text.pack_start sb, false

		textBox.pack_start buttons, false
		textBox.pack_start text
		return(textBox)
	end

	def textFrameThreadGet(board, title, dat)
		@postBox.setWindowTitle(title.chomp, @ci.server(board), board, dat)
		@title = title.chomp ; @dat = dat
		@urlview.set_text("http://#{@ci.server(board)}/test/read.cgi/#{board}/#{dat.sub(/\.dat/, '/')}")
		if @ci.bookmark?(dat)
			@bookmarkButton.child.set_text(" delete BookMark ")
		else
			@bookmarkButton.child.set_text(" add BookMark ")
		end
		@subjectLabel.set_text(" [ #{board} ] #{title.chomp}")  # #{follow}")
		@texts.freeze
		@texts.delete_text(0, @texts.get_length)
		th = @ci.getThread(board, dat)
		insertThread(@texts, th)
		@texts.thaw
		if info?(board, dat)  # ư1ܤϲΤset_valueŬѤƤʤʤǤ
			# p getVadjInfo(board, dat)
			@vadj.set_value(getVadjInfo(board, dat))
		else
			# p @vadj.value
			@vadj.set_value(0)
		end
		# p @vadj.value
		@top.set_focus @texts
	end

	def insertThread(widget, stream)
		stream.each {|line|  # from goRua. Thanks.
			case line
			when RegexID  # ȯ
				widget.insert nil, Color_DarkGreen, nil, line 
			when RegexFollowMany  # FollowUp
				widget.insert nil, nil, nil, $1
				widget.insert nil, Color_DarkBlue, nil, $2
				widget.insert nil, nil, nil, $4 + "\n"
			when RegexFollowOne  # FollowUp
				widget.insert nil, nil, nil, $1
				widget.insert nil, Color_DarkBlue, nil, $2
				widget.insert nil, nil, nil, $3 + "\n"
			when RegexQuote  # 
				widget.insert nil, Color_Purple, nil, line
			when RegexUri  # URL
				widget.insert nil, nil, nil, $1
				widget.insert nil, Color_Blue, nil, $2
				widget.insert nil, nil, nil, $3 + "\n"
			else
				widget.insert nil, nil, nil, line
			end
		}
	end

	def setThreadCached
		@subjects.each_selection {|r|
			@subjects.set_text(r, 1, 'C')
			@subjects.set_text(r, 2, '=')
			@subjects.set_text(r, 5, @subjects.get_text(r, 4))
			@ci.cachedThreadSave(@board, @subjects.get_text(r, 3))
		}
	end

	def setVadjInfo(board, dat, value)
		fh = open("#{Root}#{board}/cache/#{dat}.info", 'w')
		fh.write value.to_s
		fh.close
	end

	def getVadjInfo(board, dat)
		fh = open("#{Root}#{board}/cache/#{dat}.info")
		value = fh.read
		fh.close
		return(value.to_f)
	end

	def info?(board, dat)
		return(File.exist?("#{Root}#{board}/cache/#{dat}.info"))
	end
end

Gtk::RC.parse_string <<EOS
style "mona_12"
{
	fontset = "-mona-gothic-medium-r-normal--12-*"
}
widget_class "*" style "mona_12"
style "mona_14"
{
	fontset = "-mona-gothic-medium-r-normal--14-*"
}
widget "*Text*" style "mona_14"
EOS

w = Gtk::Window.new
w.set_default_size(525, 450)
w.set_title("C.i. - Compact Interface for 2ch. version #{CiVersion} (#{CiCode})")
w.signal_connect('delete_event') { Gtk::main_quit }
w.show_all
Pix = CiPix.new(w)
ci = Ci.new(w)

ciWidget = Gtk::HPaned.new
ciWidget.pack1(ci.mainNote, true, true)
ciWidget.pack2(ci.boardSelecter, true, true)
ciWidget.set_gutter_size(0)
ciWidget.set_handle_size(0)
ciWidget.set_position(1000)

main = Gtk::VBox.new
	buttons = Gtk::HBox.new
	select = Gtk::ToggleButton.new.add Pix.getPixmap('boardselect')
		ci.setTooltip(select, 'Board Selecter Open/Close')
		select.signal_connect('clicked') {
			if select.active?
				ciWidget.set_position(300) 
			else
				ciWidget.set_position(1000)
			end
		}
	bookmark = Gtk::ToggleButton.new.add Pix.getPixmap('bookmark')
		ci.setTooltip(bookmark, 'BookMark Open/Close')
		bookmark.signal_connect('clicked') {
			if bookmark.active?
				ci.subjectFrameUpdate(false, false, true)
			else
				ci.subjectFrameUpdate(false)
			end
		}
	reload = Gtk::Button.new.add Pix.getPixmap('refresh_blue')
		ci.setTooltip(reload, 'Reload Subjects')
		reload.signal_connect('clicked') { ci.subjectFrameUpdate }
	find = Gtk::Button.new.add Pix.getPixmap('search')
		ci.setTooltip(find, 'Thread Search')
		find.signal_connect('clicked') { ci.topNoteSetPage(1) }
	exit = Gtk::Button.new.add Pix.getPixmap('exit')
		ci.setTooltip(exit, 'Exit')
		exit.signal_connect('clicked') { ci.exitCi ; Gtk::main_quit }
	urlview = ci.urlView
	# go = Gtk::Button.new(' Go !! ')

	buttons.pack_start select, false
	buttons.pack_start bookmark, false
	buttons.pack_start reload, false
	buttons.pack_start find, false
	buttons.pack_start exit, false
	buttons.pack_start urlview, true
	# buttons.pack_start go, false

topNote = ci.topNote
topNote.prepend_page(buttons, Gtk::Label.new('Main'))

main.pack_start topNote, false
main.pack_start ciWidget

w.add main
w.show_all
Gtk.main
